# Einfache Powershellbefehle fr den AD-Papierkorb
# von Martin Lehmann

# In diesem Abschnitt versucht das Skript die Voraussetzungen zu schaffen und nicht nur einfach Fehlermeldungen zu generieren, wenn etwas fehlt. Die einfache Variante wre natrlich ber die #Requires Anweisung gewesen

if (($host).version -lt "2.0") {Throw "Bitte erst einmal Powershell 2.0 installieren. Ohne geht dieses Script nicht!"}
$admodule=$false
foreach ($i in get-module -list) {if ($i.Name -eq "ActiveDirectory") {$admodule=$true}}
if (!$admodule) {
 "Das PowershellModul fr ActiveDirectory ist auf diesem Rechner nicht vorhanden. Ich versuche es nach zu installieren."
 $srvmodule=$false
 foreach ($i in get-module -list) {if ($i.Name -eq "ServerManager") {$srvmodule=$true}}
 if (!$srvmodule) {
  Throw "Leider hat es nicht geklappt, da das Powershellmodul fr den Servermanager nicht verfgbar ist."
 } else {
  import-module ServerManager
  $adpwrshell=add-windowsfeature RSAT-AD-PowerShell 2>$nul
  if ($adpwrshell.Success -eq $false) {
    Throw "Leider konnte das Powershellmodul fr Active Directory nicht installiert werden."
  }
 }
}
$admodule=$false
foreach ($i in get-module) {if ($i.Name -eq "activedirectory") {$admodule=$true}}
if (!$admodule) {import-module activedirectory}

"`nSie haben soeben fr die aktuelle Powershell Konsole die folgenden Befehle erhalten:`n"
"Start-RecycleBin            aktiviert den Papierkorb und informiert ber den Status."
"Show-DeletedADObjects       zeigt Objekte aus dem Active Directory Papierkorb an."
"Restore-DeletedADObjects    stellt Objekte aus dem Active Directory Papierkorb nach einer Rckfrage wieder her."
"`nUm mehr ber die Befehle zu erfahren geben Sie vor dem Befehl das Wort help ein. Beispiel: help Start-RecycleBin"
# Um den Papierkorb mittels "Start-RecycleBin" anzulegen mssen Sie Organisations-Admin sein!

function Global:Start-RecycleBin {
<#
.SYNOPSIS
Aktiviert den Active Directory Papierkorb und zeigt den Status an.
.DESCRIPTION
Der Befehl untersttzt keine Parameter, da der Papierkorb nur fr die Gesamstruktur (Forest) aktiviert werden kann. Sie bentigen daher als ausfhrender Benutzer Organistionsadministratoren Berechtigung. Die Aktivierung kann nicht Rckgngig gemacht werden, da dies von Microsoft nicht vorgesehen ist!
.EXAMPLE
PS C:\Start-RecycleBin
Aktiviert den Active Directory Papierkorb.
.LINK
http://www.martinlehmann.de/wp/customizing/einfache-powershell-befehle-fur-den-active-directory-papierkorb/
.LINK
http://www.martinlehmann.de/wp/
#>
 $forest=(get-addomain).Forest
 try {
  enable-adoptionalfeature "Recycle Bin Feature" -scope ForestOrConfigurationSet -Target $forest
 }
 catch {
  write-host "`nSie haben wahrscheinlich ein Replicationsproblem im Active Directory!`nFr die Aktivierung des Papierkorbs ist dies aber nicht weiter tragisch.`nWeitere Infos liefert das Kommando: dcdiag.`n" -foregroundcolor yellow
 }
 if ((Get-ADOptionalFeature "Recycle Bin Feature").enabledscopes) {
  "Active Directory Papierkorb ist fr die Gesamtstruktur $forest verfgbar."
 } else {
  "Active Directory Papierkorb ist fr die Gesamtstruktur $forest leider nicht aktiviert."
  "Bitte prfen Sie ob Ihr Active Directory Forest auf 2008 R2 Level hochgestuft wurde."
 }
}

# "Restore-DeletedADObjects Namensteil" versucht das gelschte Objekt zu finden und wiederherzustellen

function Global:Restore-DeletedADObjects {
<#
.SYNOPSIS
Stellt gelschte Active Directory Objekte mit Hilfe eines Suchbegriffs wieder her.
.DESCRIPTION
Stellt gelschte Active Directory Objekte mit Hilfe eines Suchbegriffs wieder her. Fr den Suchbegriff mssen keine * als Platzhalter verwendet werden. Dies passiert im Script automatisch.
.PARAMETER Namensteil
Geben Sie einen Teil des Names des wiederherzustellenden Objektes ein.
.EXAMPLE
PS C:\Restore-DeletedADObjects Ben
Haben Sie beispielsweise die Benutzer mit den Namen Benutzer1,Benutzer2,Benutzer3,Benutzer4,Benutzer5,User1,User2 gelscht und Sie wrden den o.a. Befehl eingeben. Wrden Sie anschliessend fr jeden der ersten 5 Benutzer gefragt werden, ob Sie diesen Benutzer wiederherstellen mchten. User1 und 2 fallen nicht in das Suchmuster und wrden nicht zur Wiederherstellung angeboten werden.
.LINK
http://www.martinlehmann.de/wp/customizing/einfache-powershell-befehle-fur-den-active-directory-papierkorb/
.LINK
http://www.martinlehmann.de/wp/
#>
 if (!(Get-ADOptionalFeature "Recycle Bin Feature").enabledscopes) {
  Throw "Active Directory Papierkorb ist fr die Gesamtstruktur $forest nicht aktiviert. Bitte aktivieren Sie den Papierkorb zunchst mit Start-RecycleBin."
 }
 if ($args[0]) {
  $doc=$(get-addomain).DeletedObjectsContainer
  $todel=get-adobject -ldapfilter "(objectClass=*)" -searchbase $doc -includedeletedobjects | ? {$_.Name -like "*$args[0]*" -and $_.DistinguishedName -ne $doc}
  $todel
  "Wollen Sie die aufgelisteten Objekte wieder herstellen?"
  $antwort=read-host -prompt "Ja/Nein "
  if ($antwort -like "j*") {
   $todel | restore-adobject
   "Objekte sind ab sofort wieder im Active Directory verfgbar!"
  } else {
   "Objekte wurden nicht wieder hergestellt!"
  }
 } else {
  "Bitte geben Sie nach dem Befehl wenigstens einen Teil (sollte eindeutig sein) des Namens des gelschten Objektes an."
 }
}

# "Show-DeletedADObjects" zeigt alle gelschten AD Objekte an

function Global:Show-DeletedADObjects {
<#
.SYNOPSIS
Zeigt alle gelschten Active Directory Objekte an.
.DESCRIPTION
Der Befehl untersttzt keine Parameter. Alle im Papierkorb befindlichen Objekte werden ausgegeben.
.EXAMPLE
PS C:\Show-DeletedADObjects
Listet alle im Active Directory befindlichen Objekte.
.LINK
http://www.martinlehmann.de/wp/customizing/einfache-powershell-befehle-fur-den-active-directory-papierkorb/
.LINK
http://www.martinlehmann.de/wp/
#>
 if (!(Get-ADOptionalFeature "Recycle Bin Feature").enabledscopes) {
  Throw "Active Directory Papierkorb ist fr die Gesamtstruktur $forest nicht aktiviert. Bitte aktivieren Sie den Papierkorb zunchst mit Start-RecycleBin."
 }
 $doc=$(get-addomain).DeletedObjectsContainer
 $objects=get-adobject -ldapfilter "(objectClass=*)" -searchbase $doc -includedeletedobjects | ? {$_.DistinguishedName -ne $doc}
 if ($objects) {
  $objects
 } else {
  "Es sind keine gelschten Objekte vorhanden!"
  "Mglicher Weise liegt das Lschdatum zu weit in der Vergangenheit, oder der AD-Papierkorb wurde erst nach der Lschung aktiviert."
 }
}
